<?php

require DIR_APPLICATION . '../admin/model/extension/smartbill_rest.php';
require DIR_APPLICATION . '../admin/model/extension/smartbill_product.php';

class ControllerExtensionSmartbillSettings extends Controller {
	private $error = array(); // This is used to set the errors, if any.

	public function index() {
		// Default function
		$this->load->model( 'setting/setting' );
		$this->load->model( 'extension/smartbill' );
		$this->load->model( 'localisation/tax_rate' );
		$this->load->model( 'localisation/currency' );
		$this->load->model( 'localisation/order_status' );

		$this->model_extension_smartbill->validateSettingsValues();
		$data['download_url'] = $this->url->link( 'extension/smartbill_settings/download_history', 'user_token=' . $this->session->data['user_token'], 'SSL' );
		$data['update_stoc_url'] =  $this->url->link( 'extension/smartbill_settings/manual_stoc_update', 'user_token=' . $this->session->data['user_token'], 'SSL' );
		$last_date = $this->model_setting_setting->getSettingValue("SMARTBILL_STOCK_UPDATE");
		$data['last_stock_update'] = empty($last_date)?'':unserialize($last_date)->format('d/m/Y H:i');
		$this->_labels( $data );
		$this->document->setTitle( $this->language->get( 'heading_title' ) ); // Set the title of the page to the heading title in the Language file i.e., SmartBill

		if ( ! empty( $this->request->post['submitSmartBill'] ) ) {
			if ( ! $this->saveSettings() ) {
				$data['warning'] = 'Va rugam completati campurile marcate cu *.';
			} else {
				if ( $this->request->post['smartbill_send_mail_with_document'] == 1 ) {
					if ( ! empty( $this->request->post['smartbill_send_mail_bcc'] ) ) {
						if ( ! filter_var( $this->request->post['smartbill_send_mail_bcc'], FILTER_VALIDATE_EMAIL ) ) {
							$data['warning'] = 'Email-ul introdus in sectiunea Setari emitere documente nu este valid.';
						}
					}

					if ( ! empty( $this->request->post['smartbill_send_mail_cc'] ) ) {
						if ( ! filter_var( $this->request->post['smartbill_send_mail_cc'], FILTER_VALIDATE_EMAIL ) ) {
							$data['warning'] = 'Email-ul introdus in sectiunea Setari emitere documente nu este valid.';
						}
					}
				}
				if ( 'no-value' == $this->request->post['smartbill_order_unit_type'] ) {
					$data['warning'] = 'Va rugam sa selectati o unitate de masura.';
				}
				$data['success'] = 'Setarile au fost salvate';
			}
		}

		if ( $this->model_extension_smartbill->isConnected() ) {
			$data += $this->model_extension_smartbill->getSettings();

			// sync stock url
			$site_uri         = $_SERVER['REQUEST_URI'];
			$site_uri         = str_replace( 'admin/index.php', 'index.php', $site_uri );
			$site_uri         = str_replace( 'extension/smartbill_settings', 'extension/smartbill_sync_stock', $site_uri );
			$site_uri         = explode( '&user_token=', $site_uri )[0];
			$data['site_url'] = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === true ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$site_uri";

			// get vat rates
			$raw_vat_rates = json_decode( $data['SMARTBILL_TAXES'], true );
			$vat_rates     = array();
			if ( is_array( $raw_vat_rates ) ) {
				foreach ( $raw_vat_rates as $k => $v ) {
					if ( $v['percentage'] == '9999' ) {
						$vat_rates[] = array(
							'value' => $k,
							'label' => $v['name'],
						);
					} else {
						$vat_rates[] = array(
							'value' => $k,
							'label' => $v['percentage'] . '% - ' . $v['name'],
						);
					}
				}
				$data['company'] = array( 'isTaxPayer' => true );
				$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_COMPANY_IS_TAX_PAYER', 1 );
			} else {
				$vat_rates       = array(
					array(
						'value' => '',
						'label' => $raw_vat_rates,
					),
				);
				$data['company'] = array( 'isTaxPayer' => false );
				$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_COMPANY_IS_TAX_PAYER', 0 );
			}
			$data['company']['vatRates'] = $vat_rates;

			// get document series
			$invoiceSeries = $estimateSeries = array();
			$rawSeries     = json_decode( $data['SMARTBILL_SERIES'], true );
			if ( is_array( $rawSeries ) ) {
				foreach ( $rawSeries as $value ) {
					if ( $value['type'] == 'f' ) {
						$invoiceSeries[] = array(
							'label' => $value['name'],
							'value' => $value['name'],
						);
					}
					if ( $value['type'] == 'p' ) {
						$estimateSeries[] = array(
							'label' => $value['name'],
							'value' => $value['name'],
						);
					}
				}
				if ( count( $estimateSeries ) == 0 ) {
					 $estimateSeries = array(
						 array(
							 'value' => '',
							 'label' => $rawSeries,
						 ),
					 );
				}
				if ( count( $invoiceSeries ) == 0 ) {
					$invoiceSeries = array(
						array(
							'value' => '',
							'label' => $rawSeries,
						),
					);
				}
			} else {
				$invoiceSeries  = array(
					array(
						'value' => '',
						'label' => $rawSeries,
					),
				);
				$estimateSeries = array(
					array(
						'value' => '',
						'label' => $rawSeries,
					),
				);
			}

			$data['company']['invoiceSeries']  = $invoiceSeries;
			$data['company']['estimateSeries'] = $estimateSeries;

			$data['order_date'] =  [
				['value' => '0', 'label' => 'Data curenta'],
				['value' => '1', 'label' => 'Data crearii comenzii']
			];

			$data['free_shipping'] =  [
				['value' => '0', 'label' => 'Adauga transportul pe factura'],
				['value' => '1', 'label' => 'Nu adauga transportul pe factura']
			];

			// get order statuses
			$data['order_statuses']   = array();
			$data['order_statuses'][] = array(
				'value' => '0',
				'label' => 'Alege statusul',
			);
			$rawOrderStatuses         = $this->model_localisation_order_status->getOrderStatuses();
			if ( is_array( $rawOrderStatuses ) ) {
				foreach ( $rawOrderStatuses as $orderStatus ) {
					$data['order_statuses'][] = array(
						'value' => $orderStatus['order_status_id'],
						'label' => $orderStatus['name'],
					);
				}
			}

			// get measuring units
			$raw_mu = json_decode( $data['SMARTBILL_MUS'], true );
			$mu     = array(
				array(
					'value' => 'no-value',
					'label' => 'Alegeti unitatea de masura',
				),
				array(
					'value' => 'preluata-din-smartbill',
					'label' => 'Preluata din nomenclatorul SmartBill',
				),
			);
			if ( is_array( $raw_mu ) ) {
				foreach ( $raw_mu as $unit ) {
					if( 'preluata-din-smartbill' != $unit && 'no-value' != $unit ){
						$mu[] = array(
							'value' => $unit,
							'label' => $unit,
						);
					}
					
				}
			} else {
				$mu = array(
					array(
						'value' => '',
						'label' => $raw_mu,
					),
				);
			}
			$data['company']['measureUnits'] = $mu;

			// get stock
			$raw_stock = json_decode( $data['SMARTBILL_WAREHOUSES'], true );
			$stock     = array(
				array(
					'value' => '',
					'label' => 'Nu',
				),
			);
			if ( is_array( $raw_stock ) ) {

				foreach ( $raw_stock as $k => $v ) {
					$stock[] = array(
						'value' => $k,
						'label' => $v,
					);
				}
			} else {
				$stock = array(
					array(
						'value' => '',
						'label' => $raw_stock,
					),
				);
			}
			$data['company']['warehouses'] = $stock;

			// SKU types
			$data['productSKUTypes'] = $this->model_extension_smartbill->getSKUTypes();

			// Currencies
			$oc_currencies      = $this->model_localisation_currency->getCurrencies();
			$currencies         = $this->model_extension_smartbill->getCurrencies( $oc_currencies );
			$data['currencies'] = $currencies;
			$data['languages']  = $this->model_extension_smartbill->getLanguages();
			$data['used_cifs']  = $this->model_extension_smartbill->getUsedCIFs();

		} else {
			$this->response->redirect( $this->url->link( 'extension/module/smartbill', 'user_token=' . $this->session->data['user_token'], 'SSL' ) );
			exit;
		}

		$data['header']       = $this->load->controller( 'common/header' );
		$data['column_left']  = $this->load->controller( 'common/column_left' );
		$data['footer']       = $this->load->controller( 'common/footer' );
		$data['thisSettings'] = $this;
		$this->response->setOutput( $this->load->view( 'extension/module/smartbill_settings', $data ) );
	}

	public function download_history() {
		$return = array();
		$error  = '';

		try {
			$file = DIR_LOGS . 'smartbill_sincronizare_stocuri.log';
			if ( ! file_exists( $file ) ) {
				throw new Exception( 'Fisier inexistent!' );
			}

			$zip_name = 'smartbill_sincronizare_stocuri';
			$zip      = new ZipArchive();

			$zip_path = DIR_DOWNLOAD . $zip_name . '.zip';
			$zip->open( $zip_path, ZipArchive::CREATE | ZipArchive::OVERWRITE );
			$zip->addFile( $file, 'smartbill_sincronizare_stocuri.log' );
			$zip->close();

			header( 'Content-Type: application/octet-stream' );
			header( 'Content-Disposition: attachment; filename=' . basename( $zip_name ) );
			header( 'Expires: 0' );
			header( 'Cache-Control: must-revalidate' );
			header( 'Pragma: public' );
			header( 'Content-Length: ' . filesize( $zip_path ) );
			readfile( $zip_path );
			exit;

		} catch ( Exception $e ) {
			die( '' );
		}
	}

	public function download_manual_stock_history(){
		try {
			$file = DIR_DOWNLOAD. 'Istoric sincronizare manuala SmartBill.csv';
			if ( ! file_exists( $file ) ) {
				throw new Exception( 'Fisier inexistent!' );
			}

			$zip_name = 'smartbill_istoric_sincronizare_manuala_stocuri';
			$zip      = new ZipArchive();

			$zip_path = DIR_DOWNLOAD . $zip_name . '.zip';
			$zip->open( $zip_path, ZipArchive::CREATE | ZipArchive::OVERWRITE );
			$zip->addFile( $file, 'Istoric sincronizare manuala SmartBill.csv' );
			$zip->close();

			header( 'Content-Type: application/octet-stream' );
			header( 'Content-Disposition: attachment; filename=' . basename( $zip_name ) );
			header( 'Expires: 0' );
			header( 'Cache-Control: must-revalidate' );
			header( 'Pragma: public' );
			header( 'Content-Length: ' . filesize( $zip_path ) );
			readfile( $zip_path );
			exit;

		} catch ( Exception $e ) {
			die( '' );
		}
	}

	public function smartbill_get_timezone(){
		return is_null($this->config->get('config_timezone'))?$this->config->get('date_timezone'):$this->config->get('config_timezone');
	}

	public function manual_stoc_update() {
		$return = array();
		$error  = '';
		$this->load->model('setting/setting');
		$this->load->model('extension/smartbill');
		try {
			$warehouse = $this->request->get['warehouse'];
			if(!isset($warehouse) || is_null($warehouse)){	
				throw new Exception('Lipsa gestiune');
			}
			
			$now = new DateTime('now');
			$now->setTimezone(new DateTimeZone($this->smartbill_get_timezone()));
			$last_date = $this->model_setting_setting->getSettingValue("SMARTBILL_LAST_SYNC");

			if(!empty($last_date)){
				$last_date=unserialize($last_date);
				if(5 > (int)$now->diff($last_date)->i){
					$temp_date=$last_date;
					$temp_date->modify("+5 minutes");
					
					throw new Exception( sprintf( 'Preluarea manuala a stocurilor se poate face doar o data la 5 minute. Urmatoarea preluare va fi posibila la ora %s.',$temp_date->format('H:i')));
				}
			}
			
			$options = $this->model_extension_smartbill->getSettings();
			if(!isset($options['SMARTBILL_CIF']) || !isset($options['SMARTBILL_USER']) || !isset($options['SMARTBILL_API_TOKEN'])){
                throw new \Exception('Este necesar sa furnizati un utilizator si o parola valide.');
            }

			$client = new SmartBillCloudRest($options['SMARTBILL_USER'], $options['SMARTBILL_API_TOKEN']);
		
			if(empty($last_date) || 4 < (int)$now->diff($last_date)->i){
				$stocks = $client->getWareStock( $options['SMARTBILL_CIF'], $warehouse);
			}

			if(empty($stocks)){
				throw new \Exception( 'Eroare sintaxa. Verifica valididatea JSON-ului primit.' );
			}

			if(isset( $stocks['errorText'] ) && !empty( $stocks['errorText'] )){
				throw new \Exception( $stocks['errorText'] );
			}

			if ( isset( $stocks['products'] ) && 0 < count($stocks['products']) ) {
				$updated_products=0;
				
				$file = fopen(DIR_DOWNLOAD. 'Istoric sincronizare manuala SmartBill.csv', 'w');
				fputcsv($file, [ 
					'Cod produs SmartBill',
					'Denumire produs SmartBill',
					'Denumire produs Opencart',
					'ID produs Opencart',
					'Stoc nou'
				]);
				
				// Update time at the start and end of the sync.
				$this->model_setting_setting->editSettingValue( 'SMARTBILL','SMARTBILL_LAST_SYNC', serialize($now));
				
				for($i=0;$i<count($stocks['products']);$i++){
					$product = array_values($stocks['products'][$i]);
					$product[] = $options['SMARTBILL_PRODUCT_SKU_TYPE'];
					$product[] = $this->config->get('config_language_id');
					$product = new Smartbill_Product(...$product);
					
					if( true === $product->sync_quantity() ){
						$date=new DateTime('now');
						$date->setTimezone(new DateTimeZone($this->smartbill_get_timezone()));
						$this->model_setting_setting->editSettingValue('SMARTBILL', 'SMARTBILL_STOCK_UPDATE', serialize($now));
						$updated_products++;
					}

					fputcsv($file, $product->to_arr());
				}

				$product=null;
				fclose($file);
			}else{
				$error_message = sprintf( 'Nu exista produse pe Gestiunea %s. Acceseaza SmartBill Cloud, adauga produse in gestiune apoi incearca din nou preluarea stocurilor.',$warehouse);
				throw new \Exception($error_message);
			}
			
			$this->model_setting_setting->editSettingValue( 'SMARTBILL','SMARTBILL_LAST_SYNC', serialize($now));

			if( count($stocks['products']) == $updated_products ){
				$return['icon'] = 'success';
				$return['data'] = sprintf('A fost actualizat stocul pentru %s produs.', $updated_products);
				if(1 < $updated_products){
					$return['data'] = sprintf('A fost actualizat stocul pentru %s produse.', $updated_products);
				}
			}else{
				$return['icon'] = 'warning';
				$return['data'] = sprintf('A fost actualizat stocul pentru %s produs. Descarca istoricul pentru consultarea erorilor.', $updated_products);

				if( 1 < $updated_products ){
					$return['data'] = sprintf('A fost actualizat stocul pentru %s produse.  Descarca istoricul pentru consultarea erorilor.', $updated_products);
				}
				if( 0 == $updated_products ){
					$return['data'] = sprintf('Nu au fost actualizate stocurile. Descarca istoricul pentru consultarea erorilor.', $updated_products);
					$return['icon'] = 'error';
				}
			}

			echo json_encode($return);
			die();
		}catch ( Exception $e ) {
			$return['error'] = $e->getMessage();
			echo json_encode($return);
			die();
		}
	}


	public function _renderSelect( $values, $keyValue, $keyLabel, $selectedValue ) {
		$html = '';

		if ( is_array( $values ) ) {
			foreach ( $values as $item ) {
				$item     = (array) $item;
				$selected = isset( $item[ $keyValue ] ) && $item[ $keyValue ] == $selectedValue ? ' selected="selected"' : '';
				$html    .= sprintf( '<option value="%s"%s>%s</option>', $item[ $keyValue ], $selected, $item[ $keyLabel ] );
			}
		}

		return $html;
	}

	private function _labels( &$data ) {
		$this->load->language( 'extension/module/smartbill' ); // Loading the language file of smartbill

		$data['warning'] = '';
		$data['success'] = '';

		$data['heading_title']  = $this->language->get( 'heading_title' );
		$data['module_version'] = SMRT_VERSION;

		$data['text_enabled']        = $this->language->get( 'text_enabled' );
		$data['text_disabled']       = $this->language->get( 'text_disabled' );
		$data['text_content_top']    = $this->language->get( 'text_content_top' );
		$data['text_content_bottom'] = $this->language->get( 'text_content_bottom' );
		$data['text_column_left']    = $this->language->get( 'text_column_left' );
		$data['text_column_right']   = $this->language->get( 'text_column_right' );

		$data['entry_code']       = $this->language->get( 'entry_code' );
		$data['entry_layout']     = $this->language->get( 'entry_layout' );
		$data['entry_position']   = $this->language->get( 'entry_position' );
		$data['entry_status']     = $this->language->get( 'entry_status' );
		$data['entry_sort_order'] = $this->language->get( 'entry_sort_order' );

		$data['button_login']      = $this->language->get( 'button_login' );
		$data['button_save']       = $this->language->get( 'button_save' );
		$data['button_cancel']     = $this->language->get( 'button_cancel' );
		$data['button_add_module'] = $this->language->get( 'button_add_module' );
		$data['button_remove']     = $this->language->get( 'button_remove' );

		$data['action'] = $this->url->link( 'extension/smartbill_settings', 'user_token=' . $this->session->data['user_token'], 'SSL' ); // URL to be directed when the save button is pressed
		$data['cancel'] = $this->url->link( 'marketplace/extension', 'user_token=' . $this->session->data['user_token'], 'SSL' ); // URL to be redirected when cancel button is pressed
	}

	private function saveSettings() {
		$return = true;

		if ( $this->model_extension_smartbill->isConnected() ) {
			$return = $this->saveFormSettings();
		}
		return $return;
	}
	private function saveFormSettings() {
		if ( isset( $this->request->post['smartbill_prices_include_vat'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PRICES_INCLUDE_VAT', (int) $this->request->post['smartbill_prices_include_vat'] );
		}
		if ( isset( $this->request->post['smartbill_prices_vat'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PRICES_VAT', $this->request->post['smartbill_prices_vat'] );
		}
		if ( isset( $this->request->post['smartbill_transport_vat'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_TRANSPORT_VAT', $this->request->post['smartbill_transport_vat'] );
		}
		if ( isset( $this->request->post['smartbill_giftcard_vat'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_GIFTCARD_VAT', $this->request->post['smartbill_giftcard_vat'] );
		}
		if ( isset( $this->request->post['smartbill_use_payment_tax'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_USE_PAYMENT_TAX', (int) $this->request->post['smartbill_use_payment_tax'] );
		}
		if ( isset( $this->request->post['smartbill_document_type'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DOCUMENT_TYPE', $this->request->post['smartbill_document_type'] );
		}
		if ( isset( $this->request->post['smartbill_invoice_series'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_INVOICE_SERIES', $this->request->post['smartbill_invoice_series'] );
		}
		if ( isset( $this->request->post['smartbill_estimate_series'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ESTIMATE_SERIES', $this->request->post['smartbill_estimate_series'] );
		}
		if ( isset( $this->request->post['smartbill_document_date'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DOCUMENT_DATE', $this->request->post['smartbill_document_date'] );
		}		
		if ( isset( $this->request->post['smartbill_automatically_issue_document'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT', (int) $this->request->post['smartbill_automatically_issue_document'] );
		}
		if ( isset( $this->request->post['smartbill_order_status'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ORDER_STATUS', $this->request->post['smartbill_order_status'] );
		}

		if ( isset( $this->request->post['smartbill_product_sku_type'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PRODUCT_SKU_TYPE', $this->request->post['smartbill_product_sku_type'] );
		}
		if ( isset( $this->request->post['smartbill_order_unit_type'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ORDER_UNIT_TYPE', $this->request->post['smartbill_order_unit_type'] );
		}
		if ( isset( $this->request->post['smartbill_document_currency'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DOCUMENT_CURRENCY', $this->request->post['smartbill_document_currency'] );
		}
		if ( isset( $this->request->post['smartbill_document_currency_doc'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DOCUMENT_CURRENCY_DOC', $this->request->post['smartbill_document_currency_doc'] );
		}
		if ( isset( $this->request->post['smartbill_order_include_transport'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ORDER_INCLUDE_TRANSPORT', $this->request->post['smartbill_order_include_transport'] );
		}
		if ( isset( $this->request->post['smartbill_shipping_name'] ) ) {
			if(empty($this->request->post['smartbill_shipping_name'])){
				$this->request->post['smartbill_shipping_name']='Transport';
			}
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SHIPPING_NAME', $this->request->post['smartbill_shipping_name'] );
		}
		if ( isset( $this->request->post['smartbill_free_shipping'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_FREE_SHIPPING', $this->request->post['smartbill_free_shipping'] );
		}
		if ( isset( $this->request->post['smartbill_company_save_product'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_COMPANY_SAVE_PRODUCT', $this->request->post['smartbill_company_save_product'] );
		}
		if ( isset( $this->request->post['smartbill_product'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PRODUCT', $this->request->post['smartbill_product'] );
		}
		if ( isset( $this->request->post['smartbill_company_save_client'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_COMPANY_SAVE_CLIENT', $this->request->post['smartbill_company_save_client'] );
		}
		if ( isset( $this->request->post['smartbill_add_delegate_data'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ADD_DELEGATE_DATA', $this->request->post['smartbill_add_delegate_data'] );
		}
		if ( isset( $this->request->post['smartbill_issuer_name'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ISSUER_NAME', $this->request->post['smartbill_issuer_name'] );
		}
		if ( isset( $this->request->post['smartbill_issuer_cnp'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ISSUER_CNP', $this->request->post['smartbill_issuer_cnp'] );
		}
		if ( isset( $this->request->post['smartbill_delegate_name'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DELEGATE_NAME', $this->request->post['smartbill_delegate_name'] );
		}
		if ( isset( $this->request->post['smartbill_delegate_bulletin'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DELEGATE_BULLETIN', $this->request->post['smartbill_delegate_bulletin'] );
		}
		if ( isset( $this->request->post['smartbill_delegate_auto'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DELEGATE_AUTO', $this->request->post['smartbill_delegate_auto'] );
		}
		if ( isset( $this->request->post['smartbill_show_order_mention'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SHOW_ORDER_MENTION', $this->request->post['smartbill_show_order_mention'] );
		}
		if ( isset( $this->request->post['smartbill_show_order_obs'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SHOW_ORDER_OBS', $this->request->post['smartbill_show_order_obs'] );
		}
		if ( isset( $this->request->post['smartbill_custom_checkout'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CUSTOM_CHECKOUT', $this->request->post['smartbill_custom_checkout'] );
		}
		if ( isset( $this->request->post['smartbill_client_type_trans'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CLIENT_TYPE_TRANS', $this->request->post['smartbill_client_type_trans'] );
		}
		if ( isset( $this->request->post['smartbill_client_individual_trans'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CLIENT_INDIVIDUAL_TRANS', $this->request->post['smartbill_client_individual_trans'] );
		}
		if ( isset( $this->request->post['smartbill_client_entity_trans'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CLIENT_ENTITY_TRANS', $this->request->post['smartbill_client_entity_trans'] );
		}
		if ( isset( $this->request->post['smartbill_client_cif_trans'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CLIENT_CIF_TRANS', $this->request->post['smartbill_client_cif_trans'] );
		}
		if ( isset( $this->request->post['smartbill_client_reg_com_trans'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CLIENT_REG_COM_TRANS', $this->request->post['smartbill_client_reg_com_trans'] );
		}
		if ( isset( $this->request->post['smartbill_cif_vis'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CIF_VIS', $this->request->post['smartbill_cif_vis'] );
		}
		if ( isset( $this->request->post['smartbill_cif_req'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_CIF_REQ', $this->request->post['smartbill_cif_req'] );
		}
		if ( isset( $this->request->post['smartbill_reg_com_vis'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_REG_COM_VIS', $this->request->post['smartbill_reg_com_vis'] );
		}
		if ( isset( $this->request->post['smartbill_reg_com_req'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_REG_COM_REQ', $this->request->post['smartbill_reg_com_req'] );
		}

		if ( isset( $this->request->post['smartbill_price_include_discounts'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PRICE_INCLUDE_DISCOUNTS', $this->request->post['smartbill_price_include_discounts'] );
		}
		if ( isset( $this->request->post['smartbill_public_invoice'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PUBLIC_INVOICE', $this->request->post['smartbill_public_invoice'] );
		}
		if ( isset( $this->request->post['smartbill_warehouse'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_WAREHOUSE', $this->request->post['smartbill_warehouse'] );
		}
		if ( isset( $this->request->post['smartbill_payment_url'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_PAYMENT_URL', $this->request->post['smartbill_payment_url'] );
		}
		if ( isset( $this->request->post['smartbill_issue_with_due_date'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_ISSUE_WITH_DUE_DATE', $this->request->post['smartbill_issue_with_due_date'] );
		}
		if ( isset( $this->request->post['smartbill_due_days'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DUE_DAYS', $this->request->post['smartbill_due_days'] );
		}
		if ( isset( $this->request->post['smartbill_show_delivery_days'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SHOW_DELIVERY_DAYS', $this->request->post['smartbill_show_delivery_days'] );
		}
		if ( isset( $this->request->post['smartbill_delivery_days'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DELIVERY_DAYS', $this->request->post['smartbill_delivery_days'] );
		}
		if ( isset( $this->request->post['smartbill_sync_stock'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SYNC_STOCK', $this->request->post['smartbill_sync_stock'] );
		}
		if ( isset( $this->request->post['smartbill_save_stock_history'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SAVE_STOCK_HISTORY', $this->request->post['smartbill_save_stock_history'] );

			$file = DIR_LOGS . DIRECTORY_SEPARATOR . 'smartbill_sincronizare_stocuri.log';
			if ( $this->request->post['smartbill_save_stock_history'] == true && ! file_exists( $file ) ) {
				$mylog        = fopen( $file, 'a' ) or die( 'Unable to open file!' );
				$firstmessage = json_encode( '##Started Logging##' ) . "\n";
				fwrite( $mylog, $firstmessage );
				fclose( $mylog );
			}
			if ( file_exists( $file ) && $this->request->post['smartbill_save_stock_history'] == false ) {
				unlink( $file );
			}
		}
		if ( isset( $this->request->post['smartbill_used_stock'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_USED_STOCK', $this->request->post['smartbill_used_stock'] );
		}
		if ( isset( $this->request->post['smartbill_is_draft'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_IS_DRAFT', $this->request->post['smartbill_is_draft'] );
		}
		if ( isset( $this->request->post['smartbill_debug'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_DEBUG', $this->request->post['smartbill_debug'] );
		}
		if ( isset( $this->request->post['smartbill_send_mail_with_document'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SEND_MAIL_WITH_DOCUMENT', $this->request->post['smartbill_send_mail_with_document'] );
		}
		if ( isset( $this->request->post['smartbill_send_mail_cc'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SEND_MAIL_CC', $this->request->post['smartbill_send_mail_cc'] );
		}
		if ( isset( $this->request->post['smartbill_send_mail_bcc'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SEND_MAIL_BCC', $this->request->post['smartbill_send_mail_bcc'] );
		}
		if ( isset( $this->request->post['smartbill_invoice_lang'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_INVOICE_LANG', $this->request->post['smartbill_invoice_lang'] );
		}
		if ( isset( $this->request->post['smartbill_use_intra_cif'] ) ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_USE_INTRA_CIF', $this->request->post['smartbill_use_intra_cif'] );
		}

		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_TAXES', json_encode( $this->model_extension_smartbill->getVatRates() ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SERIES', json_encode( $this->model_extension_smartbill->getSeriesSettings() ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_WAREHOUSES', json_encode( $this->model_extension_smartbill->getWarehouses() ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_MUS', json_encode( $this->model_extension_smartbill->getMeasuringUnits() ) );

		$settings = $this->model_extension_smartbill->getSettings();
		if ( ! empty( $settings['SMARTBILL_COMPANY_IS_TAX_PAYER'] ) ) {
			$productsVAT    = $settings['SMARTBILL_PRICES_VAT'];
			$transportVAT   = $settings['SMARTBILL_TRANSPORT_VAT'];
			$invoiceSeries  = $settings['SMARTBILL_INVOICE_SERIES'];
			$estimateSeries = $settings['SMARTBILL_ESTIMATE_SERIES'];
			if ( ( $productsVAT == '' || $transportVAT == '' || $this->model_extension_smartbill->exportInvoice() && empty( $invoiceSeries ) )
			  || ( $this->model_extension_smartbill->exportEstimate() && empty( $estimateSeries ) ) ) {
				return false;
			}
		}

		// // Add billing custom fields
		// $this->load->model( 'customer/custom_field' );

		// $custom_fields_ids = array();
		// if ( isset( $settings['SMARTBILL_BILLING_CUSTOM_FIELDS_IDS'] ) && ! empty( $settings['SMARTBILL_BILLING_CUSTOM_FIELDS_IDS'] ) ) {
		// 	$custom_fields_ids = json_decode( $settings['SMARTBILL_BILLING_CUSTOM_FIELDS_IDS'], true );
		// }
		// // Get first customer group
		// $this->load->model( 'customer/customer_group' );
		// $customer_group     = $this->model_customer_customer_group->getCustomerGroups();
		// $customer_group_req = '';
		// if ( isset( $customer_group[0] ) && isset( $customer_group[0]['customer_group_id'] ) ) {
		// 	$customer_group     = array(
		// 		0 => array(
		// 			'customer_group_id' => $customer_group[0]['customer_group_id'],
		// 		),
		// 	);
		// 	$customer_group_req = array(
		// 		0 => array(
		// 			'customer_group_id' => $customer_group[0]['customer_group_id'],
		// 			'required'          => 1,
		// 		),
		// 	);
		// } else {
		// 	$customer_group = '';
		// }

		// $custom_field_data = array(
		// 	'custom_field_description' => array(
		// 		1 => array( 'name' => $settings['SMARTBILL_CLIENT_TYPE_TRANS'] ),
		// 	),
		// 	'location'                 => 'address',
		// 	'type'                     => 'select',
		// 	'value'                    => '33',
		// 	'validation'               => '',
		// 	'status'                   => $settings['SMARTBILL_CUSTOM_CHECKOUT'],
		// 	'sort_order'               => '',
		// 	'custom_field_value'       => array(
		// 		0 => array(
		// 			'custom_field_value_id'          => 44,
		// 			'custom_field_value_description' => array(
		// 				1 => array( 'name' => $settings['SMARTBILL_CLIENT_ENTITY_TRANS'] ),
		// 			),
		// 			'sort_order'                     => 1,
		// 		),
		// 		1 => array(
		// 			'custom_field_value_id'          => 33,
		// 			'custom_field_value_description' => array(
		// 				1 => array( 'name' => $settings['SMARTBILL_CLIENT_INDIVIDUAL_TRANS'] ),
		// 			),
		// 			'sort_order'                     => 0,
		// 		),
		// 	),
		// );

		// if ( ! empty( $customer_group_req ) ) {
		// 	$custom_field_data['custom_field_customer_group'] = $customer_group_req;
		// }

		// if ( isset( $custom_fields_ids['billing-type'] ) ) {
		// 	$billing_cf = $this->model_customer_custom_field->getCustomField( $custom_fields_ids['billing-type'] );

		// 	if ( empty( $billing_cf ) ) {
		// 		$custom_fields_ids['billing-type'] = $this->addCustomField( $custom_field_data );
		// 	}

		// 	$this->model_customer_custom_field->editCustomField( $custom_fields_ids['billing-type'], $custom_field_data );

		// } else {
		// 	$custom_fields_ids['billing-type'] = $this->addCustomField( $custom_field_data );
		// }

		// $custom_field_data = array(
		// 	'custom_field_description' => array(
		// 		1 => array( 'name' => $settings['SMARTBILL_CLIENT_CIF_TRANS'] ),
		// 	),
		// 	'location'                 => 'address',
		// 	'type'                     => 'text',
		// 	'value'                    => '',
		// 	'validation'               => '',
		// 	'status'                   => $settings['SMARTBILL_CUSTOM_CHECKOUT'] && $settings['SMARTBILL_CIF_VIS'],
		// 	'sort_order'               => '1',
		// );
		// if ( ! empty( $customer_group ) ) {
		// 	$custom_field_data['custom_field_customer_group'] = $customer_group;
		// }
		// if ( isset( $custom_fields_ids['cif'] ) ) {
		// 	$cif_cf = $this->model_customer_custom_field->getCustomField( $custom_fields_ids['cif'] );
		// 	if ( empty( $cif_cf ) ) {
		// 		$custom_fields_ids['cif'] = $this->model_customer_custom_field->addCustomField( $custom_field_data );
		// 	}
		// 	$this->model_customer_custom_field->editCustomField( $custom_fields_ids['cif'], $custom_field_data );
		// } else {
		// 	$custom_fields_ids['cif'] = $this->model_customer_custom_field->addCustomField( $custom_field_data );
		// }

		// // Nr. inregistrare Registrul Comertului
		// $custom_field_data = array(
		// 	'custom_field_description' => array(
		// 		1 => array( 'name' => $settings['SMARTBILL_CLIENT_REG_COM_TRANS'] ),
		// 	),
		// 	'location'                 => 'address',
		// 	'type'                     => 'text',
		// 	'value'                    => '',
		// 	'validation'               => '',
		// 	'status'                   => $settings['SMARTBILL_CUSTOM_CHECKOUT'] && $settings['SMARTBILL_REG_COM_VIS'],
		// 	'sort_order'               => '2',
		// );
		// if ( ! empty( $customer_group ) ) {
		// 	$custom_field_data['custom_field_customer_group'] = $customer_group;
		// }

		// if ( isset( $custom_fields_ids['reg-com'] ) ) {
		// 	$reg_cf = $this->model_customer_custom_field->getCustomField( $custom_fields_ids['reg-com'] );
		// 	if ( empty( $reg_cf ) ) {
		// 		$custom_fields_ids['reg-com'] = $this->model_customer_custom_field->addCustomField( $custom_field_data );
		// 	}
		// 	$this->model_customer_custom_field->editCustomField( $custom_fields_ids['reg-com'], $custom_field_data );
		// } else {
		// 	$custom_fields_ids['reg-com'] = $this->model_customer_custom_field->addCustomField( $custom_field_data );
		// }

		// $this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_BILLING_CUSTOM_FIELDS_IDS', json_encode( $custom_fields_ids ) );

		return true;
	}

	public function documentTypeOptions() {
		return array(
			array(
				'value' => SMRT_DOCUMENT_TYPE_INVOICE,
				'label' => 'Factura',
			),
			array(
				'value' => SMRT_DOCUMENT_TYPE_ESTIMATE,
				'label' => 'Proforma',
			),
		);
	}

	// Custom $this->model_customer_custom_field->addCustomField that forces custom value id to be set corectly
	public function addCustomField( $data ) {
		$this->db->query( 'INSERT INTO `' . DB_PREFIX . "custom_field` SET type = '" . $this->db->escape( $data['type'] ) . "', value = '" . $this->db->escape( $data['value'] ) . "', validation = '" . $this->db->escape( $data['validation'] ) . "', location = '" . $this->db->escape( $data['location'] ) . "', status = '" . (int) $data['status'] . "', sort_order = '" . (int) $data['sort_order'] . "'" );

		$custom_field_id = $this->db->getLastId();

		foreach ( $data['custom_field_description'] as $language_id => $value ) {
			$this->db->query( 'INSERT INTO ' . DB_PREFIX . "custom_field_description SET custom_field_id = '" . (int) $custom_field_id . "', language_id = '" . (int) $language_id . "', name = '" . $this->db->escape( $value['name'] ) . "'" );
		}

		if ( isset( $data['custom_field_customer_group'] ) ) {
			foreach ( $data['custom_field_customer_group'] as $custom_field_customer_group ) {
				if ( isset( $custom_field_customer_group['customer_group_id'] ) ) {
					$this->db->query( 'INSERT INTO ' . DB_PREFIX . "custom_field_customer_group SET custom_field_id = '" . (int) $custom_field_id . "', customer_group_id = '" . (int) $custom_field_customer_group['customer_group_id'] . "', required = '" . (int) ( isset( $custom_field_customer_group['required'] ) ? 1 : 0 ) . "'" );
				}
			}
		}

		if ( isset( $data['custom_field_value'] ) ) {
			foreach ( $data['custom_field_value'] as $custom_field_value ) {
				$this->db->query( 'INSERT INTO ' . DB_PREFIX . "custom_field_value SET custom_field_value_id ='" . (int) $custom_field_value['custom_field_value_id'] . "', custom_field_id = '" . (int) $custom_field_id . "', sort_order = '" . (int) $custom_field_value['sort_order'] . "'" );

				$custom_field_value_id = (int) $custom_field_value['custom_field_value_id'];

				foreach ( $custom_field_value['custom_field_value_description'] as $language_id => $custom_field_value_description ) {
					$this->db->query( 'INSERT INTO ' . DB_PREFIX . "custom_field_value_description SET custom_field_value_id = '" . (int) $custom_field_value_id . "', language_id = '" . (int) $language_id . "', custom_field_id = '" . (int) $custom_field_id . "', name = '" . $this->db->escape( $custom_field_value_description['name'] ) . "'" );
				}
			}
		}

		return $custom_field_id;
	}

}
