<?php
/**
 * @copyright  Copyright 2019-2020 © Intelligent IT SRL. All rights reserved.
 */

define( 'SMRT_VERSION', '1.4.5' );
define( 'SMRT_DOCUMENT_TYPE_INVOICE', 0 );
define( 'SMRT_DOCUMENT_TYPE_ESTIMATE', 1 );
define( 'SMRT_DATABASE_INVOICE_STATUS_DRAFT', 0 );
define( 'SMRT_DATABASE_INVOICE_STATUS_FINAL', 1 );

class ModelExtensionSmartbill extends Model {
	public $MODEL = 'model';
	public $SKU   = 'sku';
	public $UPC   = 'upc';
	public $EAN   = 'ean';
	public $JAN   = 'jan';
	public $ISBN  = 'isbn';
	public $MPN   = 'mpn';

	public function index() {
		$this->load->language( 'extension/module/smartbill' );
		$this->load->model( 'setting/setting' );
	}

	public function _get( $name ) {
		return $this->$name;
	}

	public function getSettings() {
		return $this->model_setting_setting->getSetting( 'SMARTBILL' );
	}

	public function isConnected() {
		$cif = $this->getSmartbillCif();
		return ! empty( $cif );
	}

	public function validateConnection( $settings ) {
		$status = false;
		$taxes  = false;
		try {
			$client = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );
			$taxes  = $client->getTaxes( $settings['SMARTBILL_CIF'] );

			if ( is_array( $taxes ) && isset( $taxes['taxes'] ) ) {
				$taxes['taxes'][] = array(
					'name'       => 'Preluata din OpenCart, pe produse',
					'percentage' => '9999',
				);
				$taxes            = $taxes['taxes'];

				$status = true;
			} else {
				throw new Exception( $this->language->get( 'error_connection' ) );
				$status = false;
			}
		} catch ( Exception $e ) {
			if ( $e->getMessage() == 'Firma este neplatitoare de tva.' ) {
				$status = true;
			} else {
				throw new Exception( $e->getMessage() );
				$status = false;
			}
		}

		$series = $this->getSeriesSettings( $settings['SMARTBILL_CIF'] );
		$mus    = $this->getMeasuringUnits( $settings['SMARTBILL_CIF'] );
		$wars   = $this->getWarehouses( $settings['SMARTBILL_CIF'] );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_TAXES', json_encode( $taxes ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_SERIES', json_encode( $series ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_WAREHOUSES', json_encode( $wars ) );
		$this->model_setting_setting->editSettingValue( 'SMARTBILL', 'SMARTBILL_MUS', json_encode( $mus ) );

		return $status;
	}

	private function getSmartbillCif() {
		$settings = $this->getSettings();
		return ! empty( $settings['SMARTBILL_CIF'] ) ? $settings['SMARTBILL_CIF'] : '';
	}

	public function isTaxPayer() {
		return (int) $this->config->get( 'SMARTBILL_PRICES_VAT' );
	}

	public function exportInvoice() {
		$document_type = $this->config->get( 'SMARTBILL_DOCUMENT_TYPE' );
		return $document_type == SMRT_DOCUMENT_TYPE_INVOICE;
	}
	public function exportEstimate() {
		$document_type = $this->config->get( 'SMARTBILL_DOCUMENT_TYPE' );
		return $document_type == SMRT_DOCUMENT_TYPE_ESTIMATE;
	}

	public function validateSettingsValues() {
		$settings = $this->getSettings();
		$defaults = array(
			'SMARTBILL_USER'                         => '',
			'SMARTBILL_API_TOKEN'                    => '',
			'SMARTBILL_CIF'                          => '',
			'SMARTBILL_USE_INTRA_CIF'                => '',
			'SMARTBILL_PRICES_INCLUDE_VAT'           => '1',
			'SMARTBILL_PRICES_VAT'                   => '',
			'SMARTBILL_TRANSPORT_VAT'                => '',
			'SMARTBILL_GIFTCARD_VAT'				 => '',
			'SMARTBILL_USE_PAYMENT_TAX'              => '0',
			'SMARTBILL_DOCUMENT_TYPE'                => '',
			'SMARTBILL_INVOICE_SERIES'               => '',
			'SMARTBILL_ESTIMATE_SERIES'              => '',
			'SMARTBILL_DOCUMENT_DATE'				 => '0',
			'SMARTBILL_PRODUCT_SKU_TYPE'             => '',
			'SMARTBILL_ORDER_UNIT_TYPE'              => 'no-value',
			'SMARTBILL_DOCUMENT_CURRENCY'            => '',
			'SMARTBILL_DOCUMENT_CURRENCY_DOC'        => '',
			'SMARTBILL_INVOICE_LANG'                 => '',
			'SMARTBILL_ORDER_INCLUDE_TRANSPORT'      => '1',
			'SMARTBILL_SHIPPING_NAME'				 => 'Transport',
			'SMARTBILL_FREE_SHIPPING'				 => '0',
			'SMARTBILL_COMPANY_SAVE_PRODUCT'         => '0',
			'SMARTBILL_COMPANY_SAVE_CLIENT'          => '0',
			'SMARTBILL_ADD_DELEGATE_DATA'            => '0',
			'SMARTBILL_ISSUER_NAME'   				 => '',
			'SMARTBILL_ISSUER_CNP'					 => '',
			'SMARTBILL_DELEGATE_NAME'   			 => '',
			'SMARTBILL_DELEGATE_BULLETIN'			 => '',
			'SMARTBILL_DELEGATE_AUTO'				 => '',
			'SMARTBILL_SHOW_ORDER_MENTION'			 => 'Comanda online #nr_comanda_online#',
			'SMARTBILL_SHOW_ORDER_OBS'				 => '',
			'SMARTBILL_CUSTOM_CHECKOUT'              => '0',
			'SMARTBILL_CLIENT_TYPE_TRANS'            => 'Tip Client',
			'SMARTBILL_CLIENT_INDIVIDUAL_TRANS'      => 'Persoana fizica',
			'SMARTBILL_CLIENT_ENTITY_TRANS'          => 'Persoana juridica',
			'SMARTBILL_CLIENT_CIF_TRANS'             => 'CIF',
			'SMARTBILL_CLIENT_REG_COM_TRANS'         => 'Nr. inregistrare Registrul Comertului',
			'SMARTBILL_BILLING_CUSTOM_FIELDS_IDS'    => '',
			'SMARTBILL_CIF_VIS'                      => '1',
			'SMARTBILL_CIF_REQ'                      => '1',
			'SMARTBILL_REG_COM_VIS'                  => '1',
			'SMARTBILL_REG_COM_REQ'                  => '1',
			'SMARTBILL_PRICE_INCLUDE_DISCOUNTS'      => '0',
			'SMARTBILL_PUBLIC_INVOICE'               => '0',
			'SMARTBILL_WAREHOUSE'                    => '',
			'SMARTBILL_PAYMENT_URL'					 => '0',
			'SMARTBILL_ISSUE_WITH_DUE_DATE'			 => '0',
			'SMARTBILL_DUE_DAYS'                     => '',
			'SMARTBILL_SHOW_DELIVERY_DAYS'			 => '0',
			'SMARTBILL_DELIVERY_DAYS'                => '',
			'SMARTBILL_PRODUCT'                      => '0',
			'SMARTBILL_AUTOMATICALLY_ISSUE_DOCUMENT' => '0',
			'SMARTBILL_ORDER_STATUS'                 => '',
			'SMARTBILL_COMPANY_IS_TAX_PAYER'         => '',
			'SMARTBILL_SEND_MAIL_WITH_DOCUMENT'      => '0',
			'SMARTBILL_SEND_MAIL_CC'                 => '',
			'SMARTBILL_SEND_MAIL_BCC'                => '',

			'SMARTBILL_SYNC_STOCK'                   => '0',
			'SMARTBILL_SAVE_STOCK_HISTORY'           => '0',
			'SMARTBILL_USED_STOCK'                   => '',
			'SMARTBILL_LAST_SYNC'					 => '',
			'SMARTBILL_STOCK_UPDATE'				 => '',

			'SMARTBILL_IS_DRAFT'                     => '0',
			'SMARTBILL_DEBUG'                        => '',

			'SMARTBILL_TAXES'                        => '',
			'SMARTBILL_WAREHOUSES'                   => '',
			'SMARTBILL_SERIES'                       => '',
			'SMARTBILL_MUS'                          => '',
		);

		if ( array_keys( $settings ) != array_keys( $defaults ) ) {
			$settings += $defaults;
			$this->model_setting_setting->editSetting( 'SMARTBILL', $settings );
		}
	}

	public function saveFields( $fields ) {
		if ( ! is_array( $fields ) ) {
			return;
		}
		foreach ( $fields as $field ) {
			$this->model_setting_setting->editSettingValue( 'SMARTBILL', strtoupper( $field ), trim( $this->request->post[ strtolower( $field ) ] ) );
		}
	}

	public function getFields( $fields ) {
		if ( ! is_array( $fields ) ) {
			$fields = array( $fields );
		}
		$values = array();
		foreach ( $fields as $field ) {
			$values[ strtoupper( $field ) ] = $this->model_setting_setting->getSettingValue( strtoupper( $field ) );
		}
		return $values;
	}

	/**
	 * Function used to get the VAT rates for a company
	 *
	 * @return string|array $vat_rates
	 */
	public function getVatRates( $index = -1 ) {
		$settings = $this->getSettings();

		try {
			if ( empty( $settings['SMARTBILL_CIF'] ) ) {
				return false;
			}

			$client = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );
			$taxes  = $client->getTaxes( $settings['SMARTBILL_CIF'] );

			if ( is_array( $taxes ) && isset( $taxes['taxes'] ) ) {
				$taxes['taxes'][] = array(
					'name'       => 'Preluata din OpenCart, pe produse',
					'percentage' => '9999',
				);

				if ( $index != -1 && $index != '' && is_scalar( $index ) ) {
					return $taxes['taxes'][ $index ]['percentage'];
				} else {
					return $taxes['taxes'];
				}
			} else {
				return 'Firma este neplatitoare de TVA sau nu au fost setate valori de TVA in SmartBill Cloud';
			}
		} catch ( Exception $e ) {
			return $e->getMessage();
		}

	}

	/**
	 * This function returns the invoice + estimate series from the WP DB and the SmartBill Cloud
	 *
	 * @return array
	 */
	public function getSeriesSettings( $vat_code = null ) {
		$settings = $this->getSettings();
		if ( ! is_null( $vat_code ) ) {
			$settings['SMARTBILL_CIF'] = $vat_code;
		}
		try {
			if ( empty( $settings['SMARTBILL_CIF'] ) ) {
				return false;
			}

			if ( empty( $settings['SMARTBILL_USER'] ) || empty( $settings['SMARTBILL_API_TOKEN'] ) ) {
				throw new \Exception( 'Este necesar sa furnizati un utilizator si o parola valide.' );
			}

			$connector = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );

			$raw_series = $connector->getDocumentSeries( $settings['SMARTBILL_CIF'] );

			$document_series = array();

			if ( isset( $raw_series['list'] ) && is_array( $raw_series['list'] ) ) {
				foreach ( $raw_series['list'] as $v ) {
					$document_series[] = $v;
				}
			} else {
				throw new \Exception( 'Raspuns invalid primit de la SmartBill Cloud la primirea seriilor pentru facturi.' );
			}

			return $document_series;

		} catch ( Exception $e ) {
			$error = $e->getMessage();
			return $error;
		}
	}

	/**
	 * Function used to get the measuring units for the company
	 *
	 * @return string|array $measuring_units
	 */
	public function getMeasuringUnits( $vat_code = null ) {
		$settings = $this->getSettings();

		if ( ! is_null( $vat_code ) ) {
			$settings['SMARTBILL_CIF'] = $vat_code;
		}

		try {
			if ( empty( $settings['SMARTBILL_CIF'] ) ) {
				return false;
			}

			$client = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );
			$mu     = $client->getMeasuringUnits( $settings['SMARTBILL_CIF'] );

			if ( is_array( $mu ) && isset( $mu['mu'] ) ) {
				return $mu['mu'];
			} else {
				return 'Firma este neplatitoare de TVA sau nu au fost setate valori de TVA in SmartBill Cloud';
			}
		} catch ( Exception $e ) {
			return $e->getMessage();
		}
	}

	/**
	 * Function used to get the raw currencies
	 *
	 * @return array $currencies
	 */
	public function getCurrencies( $oc_currencies, $index = -1 ) {
		$currencies = array(
			array(
				'currency' => 'RON',
				'label'    => 'RON - Leu',
			),
			array(
				'currency' => 'EUR',
				'label'    => 'EUR - Euro',
			),
			array(
				'currency' => 'USD',
				'label'    => 'USD - Dolar',
			),
			array(
				'currency' => 'GBP',
				'label'    => 'GBP - Lira sterlina',
			),
			array(
				'currency' => 'CAD',
				'label'    => 'CAD - Dolar canadian',
			),
			array(
				'currency' => 'AUD',
				'label'    => 'AUD - Dolar australian',
			),
			array(
				'currency' => 'CHF',
				'label'    => 'CHF - Franc elvetian',
			),
			array(
				'currency' => 'TRY',
				'label'    => 'TRY - Lira turceasca',
			),
			array(
				'currency' => 'CZK',
				'label'    => 'CZK - Coroana ceheasca',
			),
			array(
				'currency' => 'DKK',
				'label'    => 'DKK - Coroana daneza',
			),
			array(
				'currency' => 'HUF',
				'label'    => 'HUF - Forintul maghiar',
			),
			array(
				'currency' => 'MDL',
				'label'    => 'MDL - Leu moldovenesc',
			),
			array(
				'currency' => 'SEK',
				'label'    => 'SEK - Coroana suedeza',
			),
			array(
				'currency' => 'BGN',
				'label'    => 'BGN - Leva bulgareasca',
			),
			array(
				'currency' => 'NOK',
				'label'    => 'NOK - Coroana norvegiana',
			),
			array(
				'currency' => 'JPY',
				'label'    => 'JPY - Yenul japonez',
			),
			array(
				'currency' => 'EGP',
				'label'    => 'EGP - Lira egipteana',
			),
			array(
				'currency' => 'PLN',
				'label'    => 'PLN - Zlotul polonez',
			),
			array(
				'currency' => 'RUB',
				'label'    => 'RUB - Rubla',
			),
		);

		$e_currencies = array_column( $currencies, 'currency' );
		foreach ( $oc_currencies as $currency ) {
			if ( ! in_array( $currency['code'], $e_currencies ) ) {
				$currencies[] = array(
					'currency' => $currency['code'],
					'label'    => $currency['code'] . ' - ' . $currency['title'] . ' (Preluata din OpenCart)',
				);
			}
		}

		if ( $index != -1 && $index != '' ) {
			return $currencies[ $index ]['currency'];
		} else {
			$return = array();
			foreach ( $currencies as $k => $currency ) {
				$return[] = array(
					'value' => $k,
					'label' => $currency['label'],
				);
			}
			return $return;
		}
	}

	/**
	 * Function used to get the raw languages
	 *
	 * @return array $languages
	 */
	public function getLanguages() {
		$languages = array(
			array(
				'value' => 'RO',
				'label' => 'Romana',
			),
			array(
				'value' => 'EN',
				'label' => 'Engleza',
			),
			array(
				'value' => 'FR',
				'label' => 'Franceza',
			),
			array(
				'value' => 'IT',
				'label' => 'Italiana',
			),
			array(
				'value' => 'SP',
				'label' => 'Spaniola',
			),
			array(
				'value' => 'HU',
				'label' => 'Maghiara',
			),
			array(
				'value' => 'DE',
				'label' => 'Germana',
			),
		);

		return $languages;
	}

	/**
	 * Function used to get the used CIF options
	 *
	 * @return array $used_cifs
	 */
	public function getUsedCIFs() {
		$settings = $this->getSettings();

		$used_cifs = array(
			array(
				'value' => 'authcif',
				'label' => $settings['SMARTBILL_CIF'],
			),
			array(
				'value' => 'intracom',
				'label' => 'CIF intracomunitar',
			),
		);

		return $used_cifs;
	}

	/**
	 * Function used to get the sku types to be used
	 *
	 * @return string|array $sku_types
	 */
	public function getSKUTypes() {
		return array(
			array(
				'value' => $this->_get( 'MODEL' ),
				'label' => 'Model',
			),
			array(
				'value' => $this->_get( 'SKU' ),
				'label' => 'SKU',
			),
			array(
				'value' => $this->_get( 'UPC' ),
				'label' => 'UPC',
			),
			array(
				'value' => $this->_get( 'EAN' ),
				'label' => 'EAN',
			),
			array(
				'value' => $this->_get( 'JAN' ),
				'label' => 'JAN',
			),
			array(
				'value' => $this->_get( 'ISBN' ),
				'label' => 'ISBN',
			),
			array(
				'value' => $this->_get( 'MPN' ),
				'label' => 'MPN',
			),
		);
	}

	/**
	 * Function used to get the stocks for the company
	 *
	 * @return string|array $stocks
	 */
	public function getStock( $vat_code = null ) {
		$settings = $this->getSettings();

		if ( ! is_null( $vat_code ) ) {
			$settings['SMARTBILL_CIF'] = $vat_code;
		}

		try {
			if ( empty( $settings['SMARTBILL_CIF'] ) ) {
				return false;
			}

			$client      = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );
			$data        = array(
				'cif'           => $settings['SMARTBILL_CIF'],
				'date'          => date( 'Y-m-d' ),
				'warehouseName' => '',
				'productName'   => '',
				'productCode'   => '',
			);
			$stocks      = $client->productsStock( $data );
			$finalValues = array();
			if ( is_array( $stocks ) ) {
				foreach ( $stocks as $stock ) {
					$item['value']                 = $stock['warehouse']['warehouseName'];
					$finalValues[ $item['value'] ] = $item['value'];
				}
				return $finalValues;
			} else {
				throw new Exception( 'Raspuns invalid primit de la SmartBill Cloud la primirea informatiilor despre stocuri.' );
			}
		} catch ( Exception $e ) {
			return $e->getMessage();
		}
	}


	/**
	 * Function used to get the stocks for the company
	 *
	 * @return string|array $stocks
	 */
	public function getWarehouses( $vat_code = null ) {
		$settings = $this->getSettings();

		if ( ! is_null( $vat_code ) ) {
			$settings['SMARTBILL_CIF'] = $vat_code;
		}

		try {
			if ( empty( $settings['SMARTBILL_CIF'] ) ) {
				return false;
			}

			$client      = new SmartBillCloudRest( $settings['SMARTBILL_USER'], $settings['SMARTBILL_API_TOKEN'] );
			$warehouses  = $client->getWarehouse( $settings['SMARTBILL_CIF'] );
			$finalValues = array();

			if ( is_array( $warehouses ) ) {
				foreach ( $warehouses['warehouses'] as $warehouse ) {
					$finalValues[ $warehouse ] = $warehouse;
				}
				return $finalValues;
			} else {
				throw new Exception( 'Raspuns invalid primit de la SmartBill Cloud la primirea informatiilor despre stocuri.' );
			}
		} catch ( Exception $e ) {
			return $e->getMessage();
		}

	}


}
